var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ChangeDetectionStrategy, QueryList, ContentChildren, Input, Output, EventEmitter, HostListener, Renderer, ViewChildren } from '@angular/core';
import { SlideDirective } from './slide.directive';
import { animation, animationTimeout } from '../utils/animation-utils';
import { RebirthUIConfig } from '../rebirth-ui.config';
export var CarouselDirection;
(function (CarouselDirection) {
    CarouselDirection[CarouselDirection["NEXT"] = 0] = "NEXT";
    CarouselDirection[CarouselDirection["PREV"] = 1] = "PREV";
})(CarouselDirection || (CarouselDirection = {}));
var CarouselComponent = (function () {
    function CarouselComponent(renderer, rebirthUIConfig) {
        this.renderer = renderer;
        this.rebirthUIConfig = rebirthUIConfig;
        this.activeSlide = 0;
        this.activeSlideChange = new EventEmitter();
        this.reflowDuration = rebirthUIConfig.carousel.reflowDuration;
        this.animationDuration = rebirthUIConfig.carousel.animationDuration;
        this.animate = rebirthUIConfig.carousel.animate;
        this.interval = rebirthUIConfig.carousel.interval;
    }
    CarouselComponent.prototype.ngAfterContentInit = function () {
        if (this.interval) {
            this.startInterval();
        }
    };
    CarouselComponent.prototype.selectSlide = function (index) {
        if (this.activeSlide !== index) {
            var direction = index > this.activeSlide ? CarouselDirection.NEXT : CarouselDirection.PREV;
            this.onActiveSlideChange(index, direction);
        }
    };
    CarouselComponent.prototype.selectPrevSlide = function ($event) {
        if (this.activeSlide === 0) {
            this.onActiveSlideChange(this.slides.length - 1, CarouselDirection.PREV);
            return;
        }
        this.onActiveSlideChange(this.activeSlide - 1, CarouselDirection.PREV);
    };
    CarouselComponent.prototype.selectNextSlide = function () {
        if (this.activeSlide === this.slides.length - 1) {
            this.onActiveSlideChange(0, CarouselDirection.NEXT);
            return;
        }
        this.onActiveSlideChange(this.activeSlide + 1, CarouselDirection.NEXT);
    };
    CarouselComponent.prototype.onActiveSlideChange = function (index, carouselDirection) {
        var _this = this;
        if (this.activeSlide === index) {
            return;
        }
        this.slideAnimation(carouselDirection, index)
            .then(function () {
            _this.activeSlide = index;
            _this.activeSlideChange.emit(_this.activeSlide);
        });
    };
    CarouselComponent.prototype.pauseInterval = function () {
        this.stopInterval();
    };
    CarouselComponent.prototype.startInterval = function () {
        var _this = this;
        this.stopInterval();
        if (this.interval) {
            this.intervalId = setInterval(function () { return _this.selectNextSlide(); }, this.interval);
        }
    };
    CarouselComponent.prototype.ngOnDestroy = function () {
        this.stopInterval();
    };
    CarouselComponent.prototype.slideAnimation = function (carouselDirection, index) {
        var _this = this;
        this.stopInterval();
        var slideItems = this.slideItems.toArray();
        var direction = carouselDirection === CarouselDirection.NEXT ? 'left' : 'right';
        var orderDirection = carouselDirection === CarouselDirection.NEXT ? 'next' : 'prev';
        var nextElement = slideItems[index].nativeElement;
        var currentElement = slideItems[this.activeSlide].nativeElement;
        this.renderer.setElementClass(nextElement, orderDirection, true);
        return animationTimeout(this.reflowDuration)
            .then(function () {
            _this.renderer.setElementClass(currentElement, direction, true);
            _this.renderer.setElementClass(nextElement, direction, true);
        })
            .then(function () { return animation(_this.renderer, nextElement, _this.animationDuration); })
            .then(function () {
            _this.renderer.setElementClass(nextElement, orderDirection, false);
            _this.renderer.setElementClass(nextElement, direction, false);
            _this.renderer.setElementClass(currentElement, direction, false);
            _this.startInterval();
        });
    };
    CarouselComponent.prototype.stopInterval = function () {
        if (this.intervalId) {
            clearInterval(this.intervalId);
        }
    };
    return CarouselComponent;
}());
__decorate([
    Input(),
    __metadata("design:type", Object)
], CarouselComponent.prototype, "activeSlide", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], CarouselComponent.prototype, "interval", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], CarouselComponent.prototype, "animate", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], CarouselComponent.prototype, "activeSlideChange", void 0);
__decorate([
    ContentChildren(SlideDirective),
    __metadata("design:type", QueryList)
], CarouselComponent.prototype, "slides", void 0);
__decorate([
    ViewChildren('slideItem'),
    __metadata("design:type", QueryList)
], CarouselComponent.prototype, "slideItems", void 0);
__decorate([
    HostListener('keydown.arrowLeft', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CarouselComponent.prototype, "selectPrevSlide", null);
__decorate([
    HostListener('keydown.arrowRight', []),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CarouselComponent.prototype, "selectNextSlide", null);
__decorate([
    HostListener('mouseenter', []),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CarouselComponent.prototype, "pauseInterval", null);
__decorate([
    HostListener('mouseleave', []),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CarouselComponent.prototype, "startInterval", null);
CarouselComponent = __decorate([
    Component({
        selector: 're-carousel',
        template: "\n    <div class=\"carousel slide\" data-ride=\"carousel\" tabIndex=\"0\">\n      <ol class=\"carousel-indicators\">\n        <li *ngFor=\"let silde of slides; let $index = index\" [attr.data-slide-to]=\"$index\"\n            (click)=\"selectSlide($index)\" [ngClass]=\"{active: $index == activeSlide}\"></li>\n      </ol>\n      <div class=\"carousel-inner\" role=\"listbox\">\n        <div #slideItem *ngFor=\"let silde of slides; let $index = index\"\n             class=\"item\" [ngClass]=\"{active: $index == activeSlide}\">\n          <template [ngTemplateOutlet]=\"silde.templateRef\"></template>\n        </div>\n      </div>\n\n      <a class=\"left carousel-control\" role=\"button\" data-slide=\"prev\" (click)=\"selectPrevSlide()\">\n        <span class=\"glyphicon glyphicon-chevron-left\" aria-hidden=\"true\"></span>\n        <span class=\"sr-only\">Previous</span>\n      </a>\n      <a class=\"right carousel-control\" role=\"button\" data-slide=\"next\" (click)=\"selectNextSlide()\">\n        <span class=\"glyphicon glyphicon-chevron-right\" aria-hidden=\"true\"></span>\n        <span class=\"sr-only\">Next</span>\n      </a>\n    </div>\n  ",
        changeDetection: ChangeDetectionStrategy.OnPush,
        exportAs: 'carousel'
    }),
    __metadata("design:paramtypes", [Renderer, RebirthUIConfig])
], CarouselComponent);
export { CarouselComponent };
//# sourceMappingURL=carousel.component.js.map